.*: Assembler messages:
.*:7: Warning: Macro used \$at after "\.set noat"
.*:8: Warning: Macro used \$at after "\.set noat"
.*:9: Warning: Macro used \$at after "\.set noat"
.*:11: Warning: Macro used \$at after "\.set noat"
.*:12: Warning: Macro used \$at after "\.set noat"
.*:13: Warning: Macro used \$at after "\.set noat"
.*:14: Warning: Macro used \$at after "\.set noat"
.*:15: Warning: Macro used \$at after "\.set noat"
.*:18: Warning: Macro used \$at after "\.set noat"
.*:19: Warning: Macro used \$at after "\.set noat"
.*:20: Warning: Macro used \$at after "\.set noat"
.*:22: Warning: Macro used \$at after "\.set noat"
.*:23: Warning: Macro used \$at after "\.set noat"
.*:24: Warning: Macro used \$at after "\.set noat"
.*:25: Warning: Macro used \$at after "\.set noat"
.*:26: Warning: Macro used \$at after "\.set noat"
.*:29: Warning: Macro used \$at after "\.set noat"
.*:30: Warning: Macro used \$at after "\.set noat"
.*:31: Warning: Macro used \$at after "\.set noat"
.*:32: Warning: Macro used \$at after "\.set noat"
.*:33: Warning: Macro used \$at after "\.set noat"
.*:35: Warning: Macro used \$at after "\.set noat"
.*:36: Warning: Macro used \$at after "\.set noat"
.*:37: Warning: Macro used \$at after "\.set noat"
.*:38: Warning: Macro used \$at after "\.set noat"
.*:39: Warning: Macro used \$at after "\.set noat"
