.*: Assembler messages:
.*:21: Warning: Pretending global symbol used as branch target is local.
.*:22: Warning: Pretending global symbol used as branch target is local.
.*:23: Warning: Pretending global symbol used as branch target is local.
.*:24: Warning: Pretending global symbol used as branch target is local.
.*:25: Warning: Pretending global symbol used as branch target is local.
.*:26: Warning: Pretending global symbol used as branch target is local.
.*:35: Error: Cannot branch to undefined symbol.
.*:36: Error: Cannot branch to undefined symbol.
.*:37: Error: Cannot branch to symbol in another section.
