[^:]*: Assembler messages:
[^:]*:1: Error: conditional execution not supported with control register
[^:]*:2: Error: non-word size not supported with control register
[^:]*:3: Error: non-word size not supported with control register
[^:]*:4: Error: non-word size not supported with control register
[^:]*:5: Error: conditional execution not supported with control register
[^:]*:6: Error: non-word size not supported with control register
[^:]*:7: Error: non-word size not supported with control register
[^:]*:8: Error: non-word size not supported with control register
