.text

	.ascii ">>>>"
.global ext24
ext24:	.ascii "<<<<"
	.ascii ">>>>>"
.global ext2d
ext2d:	.ascii "<<<"
	.ascii ">>>>>>"
.global ext36
ext36:	.ascii "<<"
	.ascii ">>>>>>>"
.global ext3f
ext3f:	.ascii "<"
