.*: Assembler messages:
.*:8: Warning: Macro used \$at after "\.set noat"
.*:9: Warning: Macro used \$at after "\.set noat"
.*:11: Warning: Macro used \$at after "\.set noat"
.*:12: Warning: Macro used \$at after "\.set noat"
.*:14: Warning: Macro used \$at after "\.set noat"
.*:15: Warning: Macro used \$at after "\.set noat"
.*:17: Warning: Macro used \$at after "\.set noat"
.*:18: Warning: Macro used \$at after "\.set noat"
